#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Icon=Icon.ico
#AutoIt3Wrapper_Outfile=..\Tur-Matcher.exe
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#include <File.au3>
#include <Typos.au3>
#include <Array.au3>

#include <ButtonConstants.au3>
#include <GUIConstantsEx.au3>
#include <ProgressConstants.au3>
#include <WindowsConstants.au3>

#cs
	1.0
		* Initial release.
	1.1
		* Remove any non ASCII chars from cleaned up name. Improves match percentage.
		* Replace II with 2 etc, Up to 8. Improves match percentage.
		* Rewritten some comparison parts for a little speed upgrade. Not much though.
		* It wont ask for any user input until the end when everything is matched up.
	1.2
		* If a direct match is found, that source is removed from the array so it is not matched
		with again. Improves speed by a lot if the sourcefiles have a lot of 0 mispelled items.
		* Dupes will not be added to the source array. No reason to check the same file more then once.
	1.3
		* Now logs everything to a Logs subdir. TAB separated lists so just open them in Excel or similar.
		* You can now specify a folder full of directories as source as well. Good for artwork and similar
		that expects the files to be inside folders named after the roms/XML.
	1.4
		* Also translates &apos; to ' when reading the XML.
	1.5
		* Replaces _ with a space.
		* Replaces ' & ' with ' and '
		* Replaces ' the ' with ' '
		* Replaces '' with 'u'
		* Added traytip when its scanning source names and media so you know its not hung on really large sets. Just hover over icon.
		* Save last used settings in Tur-Matcher.ini so it defaults to the same on the next run.
		* New setting in config file, SourcesToSplit. Default is 10000. If your source media dir contains more then this amount
		of files, the array it creates to search from is split into the alphabet. Any source names will only search in the
		array that corresponds to its starting letter, greatly increasing speed if you have a LOT of source file.
		It might also mean it will miss a match if they do not share a common starting letter.
		* If the Source Name have the same name (multiple disks for example), then it will only match the first hit and reuse
		the results for the next one(s). No need to do the same search twice, or more, in a row.
	1.6
		* You now have the option to move the source media instead of copying it. An extra check if its possible will be executed if
		  you select to move.
		* Fixed saving Tur-Matcher.ini if you use an XML file. It would write the ini file in the XML file location. Oops.
#ce

Global $Version = "1.6"
Global $MediaDestination = "UNKNOWN"

Local $aArray_RomsListCleaned[0][3]
Local $aArray_SourceListCleaned[0][2]
Local $aArray_FinishedList[0][6]
Local $aArray_FinishedAutoList[0][6]
Local $aArray_NotFound[0][5]

TraySetToolTip("Tur-Matcher " & $Version)

HotKeySet("+{ESC}", "_Quit")

Global $SourcesToSplit = "10000"
Global $RomXML = ""
Global $RomXMLPath = ""
Global $RomFolder = ""
Global $HitLimit = "2"
Global $AutoCopyLimit = "0"
Global $SourceFolder = ""
Global $DestinationFolder = ""
Global $Move = "FALSE"
Global $INIFile = @ScriptDir & "/Tur-Matcher.ini"


If FileExists($INIFile) Then
	$SourcesToSplit = IniRead($INIFile, "General", "SourcesToSplit", "10000")
	$RomXML = IniRead($INIFile, "SavedSettings", "RomXML", "")
	$RomFolder = IniRead($INIFile, "SavedSettings", "RomFolder", "")
	$HitLimit = IniRead($INIFile, "SavedSettings", "HitLimit", "2")
	$AutoCopyLimit = IniRead($INIFile, "SavedSettings", "AutoCopyLimit", "0")
	$SourceFolder = IniRead($INIFile, "SavedSettings", "SourceFolder", "")
	$DestinationFolder = IniRead($INIFile, "SavedSettings", "DestinationFolder", "")
Else
	IniWrite($INIFile, "General", "SourcesToSplit", "10000")
EndIf

MsgBox(0, "Tur-Matcher " & $Version & " by Turranius", "We will now ask you a number of questions." & @CRLF & @CRLF & "Please see the title of each box that pops up for what we are asking for." & @CRLF & @CRLF & "You can press Shift-Escape at any time to quit/abort (if there is a messagebox up, you have to click on it before it exits).")

If Not FileExists("Logs") Then
	DirCreate("Logs")
EndIf

$Reply = MsgBox(32 + 4, "Use XML?", "Do you want to use an XML (database) or just use your Roms as source?" & @CRLF & @CRLF & "Press Yes for XML file or No for Roms" & @CRLF & @CRLF & @CRLF & "Please note that this does not HAVE to be the roms. This simply has to be the XML or folder that contains the names you want to use.")

If $Reply = "6" Then
	$RomXML = FileOpenDialog("Select Database XML", $RomXML, "XML files (*.XML)", 1)
	If $RomXML = "" Then
		MsgBox(16, "Error", "Got no XML location. Aborting")
		Exit
	EndIf
	; Get the path for this XML to write to INI file.
	Local $sDrive = "", $sDir = "", $sFilename = "", $sExtension = ""
	Local $aPathSplit = _PathSplit($RomXML, $sDrive, $sDir, $sFilename, $sExtension)
	$RomXMLPath = $aPathSplit[1] & $aPathSplit[2]
Else
	$RomFolder = FileSelectFolder("Please browse to the rom folder you want to use for sourcenames.", $RomFolder)
	If $RomFolder = "" Then
		MsgBox(16, "Error", "Got no Rom location. Aborting")
		Exit
	EndIf
EndIf

$HitLimit = InputBox("Threshold", "Enter the number differences that constitutes a match. Setting this too high will get false positives. Setting it to 0 will be exact match only.", $HitLimit)
If $HitLimit = "" Then
	Exit
EndIf

$AutoCopyLimit = InputBox("AutoCopy Threshold", "Enter the threashhold to use for automatic copy without questions. 0 = Only exact matches. -1 = Disable Autocopy.", $AutoCopyLimit)
If $AutoCopyLimit = "" Then
	Exit
EndIf

$SourceFolder = FileSelectFolder("Please browse to the source folder for media.", $SourceFolder)
If $SourceFolder = "" Then
	MsgBox(16, "Error", "Got no location. Aborting")
	Exit
EndIf

$DestinationFolder = FileSelectFolder("Please browse to the destination folder for media.", $DestinationFolder)
If $DestinationFolder = "" Then
	MsgBox(16, "Error", "Got no location. Aborting")
	Exit
EndIf

$MoveQuestion = MsgBox(32 + 4, "Copy matches?", "Do you want to copy or move the sourcemedia?" & @CRLF & @CRLF & "Press Yes for Copy or No for Move" & @CRLF & @CRLF & @CRLF & "Important: Only use Move for ISO files or similar where there would be no more then one use for each source media. If you select Move for other media (images), the sourcefile might be needed for more then one match." & @CRLF & "An extra check will be executed to verify if it is possible so you will get the option to change it to Copy later, should it fail the check." & @CRLF & @CRLF & "If you have the room for it, always select Yes for copy.")
If $MoveQuestion = "6" Then
	$Move = "FALSE"
Else
	$Move = "TRUE"
EndIf

If $SourceFolder = $DestinationFolder Then
	MsgBox(16, "Error", "Source and Destination folders can not be the same.")
	Exit
EndIf

IniWrite($INIFile, "SavedSettings", "AutoCopyLimit", $AutoCopyLimit)
IniWrite($INIFile, "SavedSettings", "HitLimit", $HitLimit)
IniWrite($INIFile, "SavedSettings", "SourceFolder", $SourceFolder)
IniWrite($INIFile, "SavedSettings", "DestinationFolder", $DestinationFolder)
If $RomXMLPath <> "" Then
	IniWrite($INIFile, "SavedSettings", "RomXML", $RomXMLPath)
EndIf
If $RomFolder <> "" Then
	IniWrite($INIFile, "SavedSettings", "RomFolder", $RomFolder)
EndIf


If $RomXML = "" Then ; User selected to use Roms as RomList, not the XML
	TrayTip("One moment...", "Indexing Roms. This might take a little while.", 60)
	$RomsList = _FileListToArray($RomFolder & "\")
	If @error = 1 Then
		MsgBox(0, "", "No Files Found in " & $RomFolder & "\")
		Exit
	EndIf
	$Matched = "0"
	For $i = 1 To $RomsList[0]
		TraySetToolTip("Indexing source name: " & $i)
		$RomName = $RomsList[$i]
		$RomNameNoExt = RemoveExtRegExp($RomName)
		$RomNameClean = StringRegExpReplace($RomNameNoExt, "'", '') ; Removes '
		$RomNameClean = StringRegExpReplace($RomNameClean, '_', ' ') ; Change _ to ' '
		$RomNameClean = StringRegExpReplace($RomNameClean, '|', 'u') ; Change  to u

		$RomNameClean = StringRegExpReplace($RomNameClean, ' & ', ' and ') ; Change " & " to " and "

		$RomNameClean = StringRegExpReplace(" " & $RomNameClean & " ", ' [tT][hH][eE] ', ' ') ; Change " the " to " "

		$RomNameClean = StringRegExpReplace($RomNameClean, ' III | III,', ' 3 ') ; Changes " III " and " III," to " 3 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' II | II,', ' 2 ') ; Changes " II " and " II," to " 2 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' IV | IV,', ' 4 ') ; Changes " IV " and " IV," to " 4 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VIII | VIII', ' 8 ') ; Changes " VIII " and " VIII," to " 8 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VII | VII,', ' 7 ') ; Changes " VII " and " VII," to " 7 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VI | VI,', ' 6 ') ; Changes " VI " and "VI," to " 6 "
		$RomNameClean = StringRegExpReplace($RomNameClean, ' V | V,', ' 5 ') ; Changes " V " and " VI," to " 5 "

		$RomNameClean = StringRegExpReplace($RomNameClean, '\(.*\)', '') ; Removes (*)
		$RomNameClean = StringRegExpReplace($RomNameClean, '\[.*\]', '') ; Removes [*]

		$RomNameClean = StringRegExpReplace($RomNameClean, ' III$', ' 3') ; Changes " III." to " 3."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' II$', ' 2') ; Changes " II." to " 2."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' IV$', ' 4') ; Changes " IV." to " 4."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VIII$', ' 8') ; Changes " VIII." to " 8."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VII$', ' 7') ; Changes " VII." to " 7."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' VI$', ' 6') ; Changes " VI." to " 6."
		$RomNameClean = StringRegExpReplace($RomNameClean, ' V$', ' 5') ; Changes " V." to " 5."

		$RomNameClean = StringRegExpReplace($RomNameClean, '\W', ' ') ; Removes All non ascii chars
		$RomNameClean = StringStripWS($RomNameClean, 7); Clean up extra whitespaces
		_ArrayAdd($aArray_RomsListCleaned, $RomName & "|" & $RomNameClean & "|" & $RomNameNoExt)
	Next

Else ; User selected to use a XML file as RomList.
	Local $hFileOpen = FileOpen($RomXML, 0)
	If $hFileOpen = -1 Then
		MsgBox(16, "", "An error occurred when reading the XML file.")
		Exit
	EndIf

	$Rows = _FileCountLines($RomXML)
	local $GamesFound = 0
	For $i = 1 To $Rows
		TraySetToolTip("Indexing source name from XML: " & $i)
		$line = FileReadLine($hFileOpen, $i)
		If StringInStr($line, "<game name=") Then
			$GamesFound = $GamesFound + 1
			Local $GameNameArray = StringSplit($line, '"')
			$RomName = $GameNameArray[2]
			If $RomName <> "" Then

				$RomName = StringReplace($RomName, '&amp;', '&') ; Cleanup of XML codes.
				$RomName = StringReplace($RomName, '&apos;', "'") ; Cleanup of XML codes.

				$RomNameClean = StringRegExpReplace($RomName, "'", '') ; Removes '
				$RomNameClean = StringRegExpReplace($RomNameClean, '_', ' ') ; Change _ to ' '
				$RomNameClean = StringRegExpReplace($RomNameClean, '|', 'u') ; Change  to u
				$RomNameClean = StringRegExpReplace(" " & $RomNameClean & " ", ' [tT][hH][eE] ', ' ') ; Change " the " to " "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' III | III,', ' 3 ') ; Changes " III " and " III," to " 3 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' II | II,', ' 2 ') ; Changes " II " and " II," to " 2 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' IV | IV,', ' 4 ') ; Changes " IV " and " IV," to " 4 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VIII | VIII', ' 8 ') ; Changes " VIII " and " VIII," to " 8 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VII | VII,', ' 7 ') ; Changes " VII " and " VII," to " 7 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VI | VI,', ' 6 ') ; Changes " VI " and "VI," to " 6 "
				$RomNameClean = StringRegExpReplace($RomNameClean, ' V | V,', ' 5 ') ; Changes " V " and " VI," to " 5 "

				$RomNameClean = StringRegExpReplace($RomNameClean, '\(.*\)', '') ; Removes (*)
				$RomNameClean = StringRegExpReplace($RomNameClean, '\[.*\]', '') ; Removes [*]

				$RomNameClean = StringRegExpReplace($RomNameClean, ' III$', ' 3') ; Changes " III." to " 3."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' II$', ' 2') ; Changes " II." to " 2."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' IV$', ' 4') ; Changes " IV." to " 4."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VIII$', ' 8') ; Changes " VIII." to " 8."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VII$', ' 7') ; Changes " VII." to " 7."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' VI$', ' 6') ; Changes " VI." to " 6."
				$RomNameClean = StringRegExpReplace($RomNameClean, ' V$', ' 5') ; Changes " V." to " 5."

				$RomNameClean = StringRegExpReplace($RomNameClean, '\W', ' ') ; Removes All non ascii chars
				$RomNameClean = StringStripWS($RomNameClean, 7)
				_ArrayAdd($aArray_RomsListCleaned, $RomName & "|" & $RomNameClean & "|" & $RomName)
				ConsoleWrite(StringFormat($RomName & "\n"))
				TrayTip("Indexing XML file", "Row " & $i & "/" & $Rows & " - Games found: " & $GamesFound, 5)

			EndIf
		EndIf
	Next
	TrayTip("Indexing XML file", "Row " & $Rows & "/" & $Rows, 5)
	FileClose($hFileOpen)

EndIf

_ArraySort($aArray_RomsListCleaned)
$FileName = FileOpen("Logs\SourceNames.txt", 2)
FileWrite($FileName, "Source Name" & @TAB & "Source Name Cleaned" & @TAB & "Source Name No Extensions" & @CRLF)
_FileWriteFromArray($FileName, $aArray_RomsListCleaned, 0, Default, @TAB)
FileClose($FileName)

If $Move = "TRUE" Then
	TrayTip("Verifying data.", "Verifying that Move will work!", 10)
	$RomsToCheck = UBound($aArray_RomsListCleaned) ; Check how many roms we have in the array.
	Local $LastRomNameClean = ""
	For $i = 0 To $RomsToCheck - 1 ; Here we go. Loop through every rom in the list. (-1 is because we start at 0 but $RomsToCheck start at 1.)
		$RomNameClean = $aArray_RomsListCleaned[$i][1]
		If $RomNameClean = $LastRomNameClean Then
			$ContinueAnyway = MsgBox(16 + 4, "Error!", "You have selected to MOVE the source media, but the rom " & Chr(34) &  $LastRomNameClean & Chr(34) &  " will need the same source media twice which will not work if you move it." & @CRLF & @CRLF & "Do you want to change to COPY mode instead? Selecting No will abort.")
			If $ContinueAnyway = 6 Then
				$Move = "FALSE"
			Else
				Exit
			EndIf
			ExitLoop
		EndIf
		$LastRomNameClean = $RomNameClean
	Next
EndIf

_ArrayDisplay($aArray_RomsListCleaned, $i & " Source names found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned|Source Name No Extensions", Default, 0xDDFFDD)

TrayTip("One Moment...", "Indexing source files", 5)
$SourceList = _FileListToArray($SourceFolder & "\")

$LastSourceNameClean = ""
For $i = 1 To $SourceList[0]
	TraySetToolTip("Indexing sourcemedia: " & $i)
	$SourceName = $SourceList[$i]
	$SourceNameNoExt = RemoveExtRegExp($SourceName)
	$SourceNameClean = StringRegExpReplace($SourceNameNoExt, "'", '') ; Removes '
	$SourceNameClean = StringRegExpReplace($SourceNameClean, '_', ' ') ; Change _ to ' '
	$SourceNameClean = StringRegExpReplace($SourceNameClean, '|', 'u') ; Change  to u
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' & ', ' and ') ; Change " & " to " and "
	$SourceNameClean = StringRegExpReplace(" " & $SourceNameClean & " ", ' [tT][hH][eE] ', ' ') ; Change " the " to " "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' III | III,', ' 3 ') ; Changes " III " and " III," to " 3 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' II | II,', ' 2 ') ; Changes " II " and " II," to " 2 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' IV | IV,', ' 4 ') ; Changes " IV " and " IV," to " 4 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VIII | VIII', ' 8 ') ; Changes " VIII " and " VIII," to " 8 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VII | VII,', ' 7 ') ; Changes " VII " and " VII," to " 7 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VI | VI,', ' 6 ') ; Changes " VI " and "VI," to " 6 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' V | V,', ' 5 ') ; Changes " V " and " VI," to " 5 "
	$SourceNameClean = StringRegExpReplace($SourceNameClean, '\(.*\)', '') ; Removes (*)
	$SourceNameClean = StringRegExpReplace($SourceNameClean, '\[.*\]', '') ; Removes [*]

	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' III$', ' 3') ; Changes " III." to " 3."
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' II$', ' 2') ; Changes " II." to " 2."
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' IV$', ' 4') ; Changes " IV." to " 4."

	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VIII$', ' 8') ; Changes " VIII." to " 8."
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VII$', ' 7') ; Changes " VII." to " 7."
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' VI$', ' 6') ; Changes " VI." to " 6."
	$SourceNameClean = StringRegExpReplace($SourceNameClean, ' V$', ' 5') ; Changes " V." to " 5."

	$SourceNameClean = StringRegExpReplace($SourceNameClean, '\W', ' ') ; Removes All non ascii chars
	$SourceNameClean = StringStripWS($SourceNameClean, 7)
	If $LastSourceNameClean <> $SourceNameClean Then
		_ArrayAdd($aArray_SourceListCleaned, $SourceName & "|" & $SourceNameClean)
	EndIf
	$LastSourceNameClean = $SourceNameClean
Next

TraySetToolTip("Tur-Matcher " & $Version)

_ArraySort($aArray_SourceListCleaned)

$FileName = FileOpen("Logs\SourceFiles.txt", 2)
FileWrite($FileName, "Source Name" & @TAB & "Source Name Cleaned" & @CRLF)
_FileWriteFromArray($FileName, $aArray_SourceListCleaned, 0, Default, @TAB)
FileClose($FileName)

_ArrayDisplay($aArray_SourceListCleaned, $i & " Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)

$SourcesToCheck = UBound($aArray_SourceListCleaned) - 1 ; How many sources do we have in that array?
Global $SplitArrays = "FALSE"
If $SourcesToCheck > $SourcesToSplit Then
	$SplitArrays = "TRUE"
	MsgBox(0, "Lots of sourcefiles", "We have " & $SourcesToCheck & " files as source files. This is a lot and more then the " & $SourcesToSplit & " we have set as a limit." & @CRLF & "We'll split up the sourcefiles into different arrays for each letter to speed up the search." & @CRLF & @CRLF & "Hang on!", 120)
	Local $aArray_SourceListCleaned0[0][2]
	Local $aArray_SourceListCleanedA[0][2]
	Local $aArray_SourceListCleanedB[0][2]
	Local $aArray_SourceListCleanedC[0][2]
	Local $aArray_SourceListCleanedD[0][2]
	Local $aArray_SourceListCleanedE[0][2]
	Local $aArray_SourceListCleanedF[0][2]
	Local $aArray_SourceListCleanedG[0][2]
	Local $aArray_SourceListCleanedH[0][2]
	Local $aArray_SourceListCleanedI[0][2]
	Local $aArray_SourceListCleanedJ[0][2]
	Local $aArray_SourceListCleanedK[0][2]
	Local $aArray_SourceListCleanedL[0][2]
	Local $aArray_SourceListCleanedM[0][2]
	Local $aArray_SourceListCleanedN[0][2]
	Local $aArray_SourceListCleanedO[0][2]
	Local $aArray_SourceListCleanedP[0][2]
	Local $aArray_SourceListCleanedQ[0][2]
	Local $aArray_SourceListCleanedR[0][2]
	Local $aArray_SourceListCleanedS[0][2]
	Local $aArray_SourceListCleanedT[0][2]
	Local $aArray_SourceListCleanedU[0][2]
	Local $aArray_SourceListCleanedV[0][2]
	Local $aArray_SourceListCleanedW[0][2]
	Local $aArray_SourceListCleanedX[0][2]
	Local $aArray_SourceListCleanedY[0][2]
	Local $aArray_SourceListCleanedZ[0][2]

	For $k = 0 To $SourcesToCheck ; Go through each source in the source array!
		$SourceName = $aArray_SourceListCleaned[$k][0] ; Get the sourcename
		$SourceNameClean = $aArray_SourceListCleaned[$k][1] ; Get the cleaned SourceName
		Local $Letter = StringUpper(StringLeft($SourceNameClean, 1))
		Switch $Letter
			Case "A"
				_ArrayAdd($aArray_SourceListCleanedA, $SourceName & "|" & $SourceNameClean)
			Case "B"
				_ArrayAdd($aArray_SourceListCleanedB, $SourceName & "|" & $SourceNameClean)
			Case "C"
				_ArrayAdd($aArray_SourceListCleanedC, $SourceName & "|" & $SourceNameClean)
			Case "D"
				_ArrayAdd($aArray_SourceListCleanedD, $SourceName & "|" & $SourceNameClean)
			Case "E"
				_ArrayAdd($aArray_SourceListCleanedE, $SourceName & "|" & $SourceNameClean)
			Case "F"
				_ArrayAdd($aArray_SourceListCleanedF, $SourceName & "|" & $SourceNameClean)
			Case "G"
				_ArrayAdd($aArray_SourceListCleanedG, $SourceName & "|" & $SourceNameClean)
			Case "H"
				_ArrayAdd($aArray_SourceListCleanedH, $SourceName & "|" & $SourceNameClean)
			Case "I"
				_ArrayAdd($aArray_SourceListCleanedI, $SourceName & "|" & $SourceNameClean)
			Case "J"
				_ArrayAdd($aArray_SourceListCleanedJ, $SourceName & "|" & $SourceNameClean)
			Case "K"
				_ArrayAdd($aArray_SourceListCleanedK, $SourceName & "|" & $SourceNameClean)
			Case "L"
				_ArrayAdd($aArray_SourceListCleanedL, $SourceName & "|" & $SourceNameClean)
			Case "M"
				_ArrayAdd($aArray_SourceListCleanedM, $SourceName & "|" & $SourceNameClean)
			Case "N"
				_ArrayAdd($aArray_SourceListCleanedN, $SourceName & "|" & $SourceNameClean)
			Case "O"
				_ArrayAdd($aArray_SourceListCleanedO, $SourceName & "|" & $SourceNameClean)
			Case "P"
				_ArrayAdd($aArray_SourceListCleanedP, $SourceName & "|" & $SourceNameClean)
			Case "Q"
				_ArrayAdd($aArray_SourceListCleanedQ, $SourceName & "|" & $SourceNameClean)
			Case "R"
				_ArrayAdd($aArray_SourceListCleanedR, $SourceName & "|" & $SourceNameClean)
			Case "S"
				_ArrayAdd($aArray_SourceListCleanedS, $SourceName & "|" & $SourceNameClean)
			Case "T"
				_ArrayAdd($aArray_SourceListCleanedT, $SourceName & "|" & $SourceNameClean)
			Case "U"
				_ArrayAdd($aArray_SourceListCleanedU, $SourceName & "|" & $SourceNameClean)
			Case "V"
				_ArrayAdd($aArray_SourceListCleanedV, $SourceName & "|" & $SourceNameClean)
			Case "W"
				_ArrayAdd($aArray_SourceListCleanedW, $SourceName & "|" & $SourceNameClean)
			Case "X"
				_ArrayAdd($aArray_SourceListCleanedX, $SourceName & "|" & $SourceNameClean)
			Case "Y"
				_ArrayAdd($aArray_SourceListCleanedY, $SourceName & "|" & $SourceNameClean)
			Case "Z"
				_ArrayAdd($aArray_SourceListCleanedZ, $SourceName & "|" & $SourceNameClean)
			Case 0 To 9
				_ArrayAdd($aArray_SourceListCleaned0, $SourceName & "|" & $SourceNameClean)
		EndSwitch
	Next

	;Local $aArray_SourceListCleanedTemp[0][2]
	;Local $TestVar = "F"
	;	$aArray_SourceListCleanedTemp = $aArray_SourceListCleaned & $TestVar
	;	_ArrayDisplay($aArray_SourceListCleanedTemp, " Temp", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)

	#cs
		_ArrayDisplay($aArray_SourceListCleaned0, " 0 Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedA, " A Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedB, " B Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedC, " C Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedD, " D Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedE, " E Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedF, " F Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedG, " G Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedH, " H Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedI, " I Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedJ, " J Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedK, " K Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedL, " L Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedM, " M Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedN, " N Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedO, " O Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedP, " P Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedQ, " Q Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedR, " R Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedS, " S Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedT, " T Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedU, " U Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedV, " V Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedW, " W Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedX, " X Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedY, " Y Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
		_ArrayDisplay($aArray_SourceListCleanedZ, " Z Source files Found. Close this window on the upper X to continue.", "", 16, -1, "Source Name|Source Name Cleaned", Default, 0xDDFFDD)
	#ce
EndIf

Global $Letter
Global $LastLetter
Global $LastRomNameClean
Global $LastbestMatch
Global $LastBestMatchNameCleaned
Global $LastBestMatchName
Global $LastbestMatchIdx
Global $bestMatchIdx
Global $bestMatch
Global $BestMatchNameExt


$Runs = 0
$RomsToCheck = UBound($aArray_RomsListCleaned) ; Check how many roms we have in the array.
TraySetToolTip("Tur-Matcher starting matching...")
For $i = 0 To $RomsToCheck - 1 ; Here we go. Loop through every rom in the list. (-1 is because we start at 0 but $RomsToCheck start at 1.)
	$Skip = "FALSE" ; Set this to false. We will do some checks and then we can set it to TRUE to have it skipped later.
	$RomNameNoExtension = $aArray_RomsListCleaned[$i][2] ; Get the RomName with no extension.

	TraySetToolTip("Tur-Matcher matching " & $RomNameNoExtension & " (" & $i & "/" & $RomsToCheck & ")")
	If FileExists($DestinationFolder & "\" & $RomNameNoExtension & ".*") Then ; If this destination file already exists (actually checks .* so extension does not matter).
		ConsoleWrite(StringFormat("Skipping " & $RomNameNoExtension & " as it already exists in " & $DestinationFolder & "\n"))
		$Skip = "TRUE" ; Destination file already exists so we can skip this item.
	EndIf

	If $Skip = "FALSE" Then ; Should we continue with this item? No use getting these values otherwise.
		$RomName = $aArray_RomsListCleaned[$i][0]
		$RomNameClean = $aArray_RomsListCleaned[$i][1] ; Get the romname
		ConsoleWrite(StringFormat($Runs & ": Checking " & $RomNameClean & " -> "))
	EndIf

	; If this is the same romname as the last one, use the same source. No need to search again.
	If $Skip = "FALSE" Then ; Should we continue with this item?
		If $RomNameClean = $LastRomNameClean Then
			$Skip = "TRUE"
			ConsoleWrite(StringFormat("SAME match (Array: " & $Letter & "): " & $LastBestMatchName & " (" & $LastbestMatchIdx & ") with " & $LastbestMatch & " spelling errors\n"))
			TrayTip($i & "/" & $RomsToCheck & " - Same match for " & $RomNameClean, $LastBestMatchNameCleaned & " (" & $LastbestMatch & ") Sources (" & $Letter & "):" & $SourcesToCheck, 5)
			If $HitLimit + 1 > $bestMatch Then
				If $AutoCopyLimit >= $LastbestMatch Then
					_ArrayAdd($aArray_FinishedAutoList, $RomNameClean & "|" & $LastBestMatchNameCleaned & "|" & $RomName & "|" & $LastBestMatchName & "|" & $LastbestMatch & "|" & $RomNameNoExtension & $BestMatchNameExt) ; Autocopy stuff.
				Else
					_ArrayAdd($aArray_FinishedList, $RomNameClean & "|" & $LastBestMatchNameCleaned & "|" & $RomName & "|" & $LastBestMatchName & "|" & $LastbestMatch & "|" & $RomNameNoExtension & $BestMatchNameExt) ; Manual stuff we'll go through later.
				EndIf
			Else
				_ArrayAdd($aArray_NotFound, $RomNameClean & "|" & $LastBestMatchNameCleaned & "|" & $RomName & "|" & $LastBestMatchName & "|" & $LastbestMatch) ; Stuff that did not make the cut at all.
			EndIf
		EndIf
	EndIf

	If $Skip = "FALSE" Then ; Should we continue with this item?
		$Runs = $Runs + 1
		; This is debug if we want to only run a portion of the roms.
		;	  If $Runs > "50" Then
		;		ExitLoop
		;	  EndIf

		; Reset these values for each item we go through.
		$ExactHit = "FALSE"
		$bestMatch = ""
		$bestMatchIdx = ""



		Local $bestMatch = StringLen($RomNameClean) ; Get the lenght of the name we're going to test first. We'll compare the results to this number.

		If $SplitArrays = "TRUE" Then
			Local $Letter = StringUpper(StringLeft($RomNameClean, 1)); Get the first letter of the rom.

			; Always 0 if its 0-9 as they are all in the same array.
			If StringIsDigit($Letter) Then
				$Letter = "0"
			EndIf

			If $LastLetter <> $Letter Then
				TrayTip("Tur-Matcher", "Switching array to the letter " & $Letter, 5)
				Sleep(2000)
				Switch $Letter
					Case "A"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedA
					Case "B"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedB
					Case "C"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedC
					Case "D"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedD
					Case "E"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedE
					Case "F"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedF
					Case "G"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedG
					Case "H"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedH
					Case "I"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedI
					Case "J"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedJ
					Case "K"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedK
					Case "L"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedL
					Case "M"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedM
					Case "N"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedN
					Case "O"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedO
					Case "P"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedP
					Case "Q"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedQ
					Case "R"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedR
					Case "S"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedS
					Case "T"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedT
					Case "U"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedU
					Case "V"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedV
					Case "W"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedW
					Case "X"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedX
					Case "Y"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedY
					Case "Z"
						$aArray_SourceListCleaned = $aArray_SourceListCleanedZ
					Case 0 To 9
						$aArray_SourceListCleaned = $aArray_SourceListCleaned0
				EndSwitch
			EndIf
		EndIf

		$SourcesToCheck = UBound($aArray_SourceListCleaned) - 1 ; How many sources do we have in that array?
		For $k = 0 To $SourcesToCheck ; Go through each source in the source array!
			$SourceNameClean = $aArray_SourceListCleaned[$k][1] ; Get the cleaned SourceName
			$typos = _Typos($SourceNameClean, $RomNameClean) ; Run a comparison on SourceNameClean and RomNameClean. It will return the differences between them.
			If $typos < $bestMatch Then ; If this difference is lower then the previous one, its a metter match and we need to update $bestMatch and $bestMatchIdx with the better hit.
				$bestMatch = $typos
				$bestMatchIdx = $k

				If $typos = 0 Then ; If $typos is 0, it means it an exact hit.
					$ExactHit = "TRUE" ; We need this for later.
					ExitLoop ; Exit the loop through the sources. No need to if we found a perfect match already. Speeds things up.
				EndIf
			EndIf
		Next

		; Nothing found...
		If $bestMatchIdx = "" And $ExactHit = "FALSE" Then
			$bestMatchIdx = "NA"
			$BestMatchName = "Nothing"
			$bestMatch = "Nothing"
			ConsoleWrite(StringFormat("No hit for " & $RomName & "\n"))
			TrayTip("No hit for " & $RomName, "", 5)
		Else
			$BestMatchName = $aArray_SourceListCleaned[$bestMatchIdx][0] ; Get the best matched fullname
			$BestMatchNameExt = _FileGetExt($BestMatchName) ; Get the extension from the best matched fullname. That will be the destination extension.
			$BestMatchNameCleaned = $aArray_SourceListCleaned[$bestMatchIdx][1] ; Also get the best matched cleaned name.
			If $SplitArrays = "FALSE" Then
				ConsoleWrite(StringFormat("Best match : " & $BestMatchName & " (" & $bestMatchIdx & ") with " & $bestMatch & " spelling errors\n"))
				TrayTip($i & "/" & $RomsToCheck & " - Best match for " & $RomNameClean, $BestMatchNameCleaned & " (" & $bestMatch & ") Sources:" & $SourcesToCheck, 5)

			Else
				ConsoleWrite(StringFormat("Best match (Array: " & $Letter & "): " & $BestMatchName & " (" & $bestMatchIdx & ") with " & $bestMatch & " spelling errors\n"))
				TrayTip($i & "/" & $RomsToCheck & " - Best match for " & $RomNameClean, $BestMatchNameCleaned & " (" & $bestMatch & ") Sources (" & $Letter & "):" & $SourcesToCheck, 5)
			EndIf
			If $HitLimit + 1 > $bestMatch Then
				If $AutoCopyLimit >= $bestMatch Then
					_ArrayAdd($aArray_FinishedAutoList, $RomNameClean & "|" & $BestMatchNameCleaned & "|" & $RomName & "|" & $BestMatchName & "|" & $bestMatch & "|" & $RomNameNoExtension & $BestMatchNameExt) ; Autocopy stuff.
				Else
					_ArrayAdd($aArray_FinishedList, $RomNameClean & "|" & $BestMatchNameCleaned & "|" & $RomName & "|" & $BestMatchName & "|" & $bestMatch & "|" & $RomNameNoExtension & $BestMatchNameExt) ; Manual stuff we'll go through later.
				EndIf
			Else
				_ArrayAdd($aArray_NotFound, $RomNameClean & "|" & $BestMatchNameCleaned & "|" & $RomName & "|" & $BestMatchName & "|" & $bestMatch) ; Stuff that did not make the cut at all.
			EndIf
			$LastRomNameClean = $RomNameClean
			$LastbestMatchIdx = $bestMatchIdx
			$LastbestMatch = $bestMatch
			$LastBestMatchNameCleaned = $BestMatchNameCleaned
			$LastBestMatchName = $BestMatchName
		EndIf

		If $ExactHit = "TRUE" Then ; If this was a direct hit (0), we can remove this hit from the Sourcenames array to speed up search.
			If $SourcesToCheck > "50" Then ; No point in removing anything from the sources array if its less then 50 items in it.
				If $i + 1 < $RomsToCheck Then ; Make sure there are one more item in coming up in the rom array.
					$RomNameNextClean = $aArray_RomsListCleaned[$i + 1][1] ; Grab the next rom thats coming up.
					If $RomNameClean <> $RomNameNextClean Then ; If the current rom is NOT the same as the next rom, we can remove this source from the source array to speed up searches.
						_ArrayDelete($aArray_SourceListCleaned, $bestMatchIdx); Delete this item from array.
					EndIf
				EndIf
			EndIf
		EndIf

	EndIf

	If $SplitArrays = "TRUE" Then
		$LastLetter = $Letter
	EndIf

Next
TraySetToolTip("Tur-Matcher " & $Version)

$FileName = FileOpen("Logs\AutoCopy.txt", 2)
FileWrite($FileName, "Source Name Clean" & @TAB & "Source File Clean" & @TAB & "Source Full Name" & @TAB & "Source File Full Name" & @TAB & "Misspellings" & @TAB & "New Name" & @CRLF)
_FileWriteFromArray($FileName, $aArray_FinishedAutoList, 0, Default, @TAB)
FileClose($FileName)

_ArrayDisplay($aArray_FinishedAutoList, "Here are the items that will now be automatically copied. Press X to continue.", "", 16, -1, "Source Name Clean|Source File Clean|Source Full Name|Source File Full Name|Misspellings|New Name", Default, 0xDDFFDD)

TrayTip("Copying", "Copying files within the autocopy limit. Hang on.", 5)

For $i = 0 To UBound($aArray_FinishedAutoList) - 1

	$RomNameClean = $aArray_FinishedAutoList[$i][0]
	$BestMatchNameCleaned = $aArray_FinishedAutoList[$i][1]
	$RomName = $aArray_FinishedAutoList[$i][2]
	$BestMatchName = $aArray_FinishedAutoList[$i][3]
	$bestMatch = $aArray_FinishedAutoList[$i][4]
	$NewName = $aArray_FinishedAutoList[$i][5]

	If Not FileExists($DestinationFolder) Then
		MsgBox(16, "Failure", "Could not find destinationfolder" & @CRLF & $DestinationFolder & @CRLF & @CRLF & "Aborting.")
		Exit
	Else
		$DirOrFile = FileGetAttrib($SourceFolder & "\" & $BestMatchName)
		If StringInStr($DirOrFile, "D") Then
			If $Move = "TRUE" Then
				TrayTip("Dir: " & $NewName, "Moving files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("Dir Automove: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				If FileExists($SourceFolder & "\" & $BestMatchName) Then
					DirMove($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
				Else
					TrayTip("Dir: " & $NewName, $SourceFolder & "\" & $BestMatchName & " No longer exists because it was moved previously.. skipping.", 5)
					Sleep(2000)
				EndIf
			Else
				TrayTip("Dir: " & $NewName, "Copying files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("Dir Autocopy: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				DirCopy($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
			EndIf
		Else
			If $Move = "TRUE" Then
				TrayTip("File: " & $NewName, "Moving files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("File Automove: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				If FileExists($SourceFolder & "\" & $BestMatchName) Then
					FileMove($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
				Else
					TrayTip("Dir: " & $NewName, $SourceFolder & "\" & $BestMatchName & " No longer exists because it was moved previously.. skipping.", 5)
					Sleep(2000)
				EndIf
			Else
				TrayTip("File: " & $NewName, "Copying files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("File Autocopy: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				FileCopy($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
			EndIf
		EndIf
	EndIf
Next

$FileName = FileOpen("Logs\ManualCopy.txt", 2)
FileWrite($FileName, "Source Name Clean" & @TAB & "Source File Clean" & @TAB & "Source Full Name" & @TAB & "Source File Full Name" & @TAB & "Misspellings" & @TAB & "New Name" & @CRLF)
_FileWriteFromArray($FileName, $aArray_FinishedList, 0, Default, @TAB)
FileClose($FileName)

_ArrayDisplay($aArray_FinishedList, "Here are the items that we will go over now. Press X to continue.", "", 16, -1, "Source Name Clean|Source File Clean|Source Full Name|Source File Full Name|Misspellings|New Name", Default, 0xDDFFDD)

For $i = 0 To UBound($aArray_FinishedList) - 1
	$RomNameClean = $aArray_FinishedList[$i][0]
	$BestMatchNameCleaned = $aArray_FinishedList[$i][1]
	$RomName = $aArray_FinishedList[$i][2]
	$BestMatchName = $aArray_FinishedList[$i][3]
	$bestMatch = $aArray_FinishedList[$i][4]
	$NewName = $aArray_FinishedList[$i][5]

	$Reply = MsgBox(3 + 32, "Is this a match?", "Would you say that these match?" & @CRLF & @CRLF & $RomNameClean & @CRLF & $BestMatchNameCleaned & @CRLF & @CRLF & "Exact names are:" & @CRLF & $RomName & @CRLF & $BestMatchName & @CRLF & @CRLF & "Misspellings: " & $bestMatch)
	If $Reply = "2" Then
		Exit
	EndIf
	If $Reply = "7" Then
		ConsoleWrite(StringFormat("Skipping: " & $BestMatchName & "\n"))
	EndIf
	If $Reply = "6" Then
		$DirOrFile = FileGetAttrib($SourceFolder & "\" & $BestMatchName)

		If StringInStr($DirOrFile, "D") Then
			If $Move = "TRUE" Then
				TrayTip("Dir: " & $NewName, "Moving files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("Manual File move: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				If FileExists($SourceFolder & "\" & $BestMatchName) Then
					DirMove($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
				Else
					TrayTip("Dir: " & $NewName, $SourceFolder & "\" & $BestMatchName & " No longer exists because it was moved previously.. skipping.", 5)
					Sleep(2000)
				EndIf
			Else
				TrayTip("Dir: " & $NewName, "Copying files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("Dir Autocopy: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				DirCopy($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
			EndIf
		Else
			If $Move = "TRUE" Then
				TrayTip("File: " & $NewName, "Moving files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("Manual File copy: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				If FileExists($SourceFolder & "\" & $BestMatchName) Then
					FileMove($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
				Else
					TrayTip("Dir: " & $NewName, $SourceFolder & "\" & $BestMatchName & " No longer exists because it was moved previously.. skipping.", 5)
					Sleep(2000)
				EndIf
			Else
				TrayTip("File: " & $NewName, "Copying files within the autocopy limit. Hang on.", 5)
				ConsoleWrite(StringFormat("File Autocopy: " & $SourceFolder & "\" & $BestMatchName & "  ->  " & $DestinationFolder & "\" & $NewName & "\n"))
				FileCopy($SourceFolder & "\" & $BestMatchName, $DestinationFolder & "\" & $NewName)
			EndIf
		EndIf

	EndIf
Next

$FileName = FileOpen("Logs\NoMatch.txt", 2)
FileWrite($FileName, "Source Name Clean" & @TAB & "Source File Clean" & @TAB & "Source Full Name" & @TAB & "Source File Full Name" & @TAB & "Misspellings" & @CRLF)
_FileWriteFromArray($FileName, $aArray_NotFound, 0, Default, @TAB)
FileClose($FileName)

_ArrayDisplay($aArray_NotFound, "Here are the best matches for unmatched media. Press X to quit the program.", "", 16, -1, "Source Name Clean|Best File Clean|source Full Name|Best File Full Name|Misspellings", Default, 0xDDFFDD)

MsgBox(0, "Done", "All done!")

Func RemoveExtRegExp($Input)
	Return StringRegExpReplace($Input, "\.[^.]*$", "")
EndFunc   ;==>RemoveExtRegExp

Func _FileGetExt($sPath)
	Local $NULL, $sExt
	_PathSplit($sPath, $NULL, $NULL, $NULL, $sExt)
	Return $sExt
EndFunc   ;==>_FileGetExt

; This function is called whenever you quit the script.
Func _Quit()
	Exit
EndFunc   ;==>_Quit
